"""
Messages found throughout Cabinet
"""

import pathlib

NEW_SETUP_MSG_INTRO = (
    "Welcome to Cabinet!\n\n"
    "Do you have a MongoDB instance set up? (y/n)\n"
)

NEW_SETUP_MSG_MONGODB_INSTRUCTIONS = """
Cabinet will use MongoDB to store and manage data.
Don't worry, this is easy and free for our purposes.

To create a MongoDB account using the free tier, follow these instructions:

1. Visit the MongoDB website: https://www.mongodb.com/
2. Click on the "Try Free" button.
3. Sign up for a free MongoDB account by providing the required information.
    - If prompted to create a new project, give any name you'd like.
4. Once you have created your account and logged in, create a new cluster.
    - Give the cluster any name you'd like.
5. Create a new database in your cluster with the name of your choice.
    - Name the collection "cabinet".
    - Unless you have a specific reason, don't add "additional preferences".

Press Enter once this is done.\n\n
"""

CONFIG_MONGODB_USERNAME = """
Enter your MongoDB username (not your email address):\n
"""

CONFIG_MONGODB_PASSWORD = """
Enter your MongoDB password:\n
"""

CONFIG_MONGODB_CLUSTER_NAME = """
Enter your MongoDB cluster name:\n
"""

CONFIG_MONGODB_DB_NAME = """
Enter your MongoDB database name:\n
"""

CONFIG_PATH_CABINET = f"""
Enter the full path where you would like to store Cabinet data,
such as logs and settings.

\n
Default: {pathlib.Path.home().resolve()}/.cabinet
"""

CONFIG_EDITOR: str = """
Select your preferred editor from the list.
\n\n
If you're not sure, just hit Enter.
"""

EDIT_FILE_DEFAULT = """
Enter the path of the file you want to edit.
(default: edit Cabinet's MongoDB collection):\n
"""

ERROR_CONFIG_FILE_INVALID = """
Cabinet could not initialize properly.

Please check that all values in the configuration file are correct:
mongodb_username
mongodb_password
mongodb_cluster_name
mongodb_db_name
path_cabinet
editor

Please try re-running Cabinet.

Otherwise, please leave feedback at https://github.com/tylerjwoodfin/cabinet/issues.\n\n
"""

ERROR_CONFIG_MISSING_VALUES = """
Cabinet could not initialize properly- some values appear to be missing.

Please check that all values in the configuration file exist and are properly set:
mongodb_username
mongodb_password
mongodb_cluster_name
mongodb_db_name
path_cabinet
editor

Press Enter to open the file. Update any invalid values and try again.\n
"""

ERROR_CONFIG_JSON_DECODE = """
The configuration file is not valid JSON.

Do you want to replace it with an empty JSON file?
This will reset your MongoDB credentials, but your data will be safe.

(y/n)\n
"""

ERROR_MONGODB_TIMEOUT = """
Timeout error: Could not connect to the MongoDB server:
"""

ERROR_MONGODB_DNS = """
DNS resolution failed for MongoDB server:
"""

ERROR_CONFIG_INVALID_EDITOR = """
I didn't understand that.

Your editor will be set to 'nano'.

Change this at any time with `cabinet --config`.
"""

ERROR_CONFIG_BROKEN_EDITOR = """
Cabinet's editor is set to %, but this editor cannot be used.

Please update the editor by using `cabinet --config` and updating
the editor attribute.

Your editor has been reset to 'nano', so if you try your last command
again, you shouldn't see this error.
"""


"""
Cabinet Mail

Provides functionality for sending email using SMTP and MIMEText.
Does not support Gmail.

A throwaway email is highly recommended.
"""

import smtplib
from urllib.parse import unquote
import subprocess
import socket
import shlex
import pwd
import sys
import os
from typing import List
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
import cabinet

class Mail:
    """
    Provides functionality for sending email using SMTP and MIMEText.
    Does not support Gmail.
    A throwaway email is highly recommended.
    """

    def __init__(self):
        self.user_dir = pwd.getpwuid(os.getuid())[0]
        self.cab = cabinet.Cabinet()

        # parameters
        # port should be int; TODO update Cabinet to support type casting
        self.port = self.cab.get("email", "port")

        self.smtp_server: str | None = self.cab.get("email", "smtp_server")
        self.imap_server: str | None = self.cab.get("email", "imap_server")
        self.username = self.cab.get("email", "from")
        self.password = self.cab.get("email", "from_pw")

    def send(self, subject: str,
             body: str,
             signature: str = '',
             to_addr = None, # should be List[str]; TODO update Cabinet to support type casting
             from_name: str | None = None,
             logging_enabled: bool = True,
             is_quiet: bool = False) -> None:
        """
        Sends an email with the given subject and body to the specified recipients.

        Args:
        - subject (str): The subject of the email.
        - body (str): The body of the email.
        - signature (str): The signature to include at the end of the email.
        - to_addr (List): A list of email addresses to send the email to.
        - from_name (str, optional): The name to appear in the "From" field of the email.
            Reads from cabinet -> email -> from_name if unset.
            If this is unset, defaults to machine's hostname or `Cabinet`
        - logging_enabled (bool, optional): Whether to log the email send event.
            Defaults to True.
        - is_quiet: Whether to suppress log output.
            Defaults to False.

        Raises:
        - smtplib.SMTPAuthenticationError: If the SMTP server rejects the login credentials.

        Gmail will almost certainly not work.
        """

        hostname = socket.gethostname()
        if hostname:
            hostname = hostname.capitalize().replace(".local", "")

        cab_from_name = self.cab.get("email", "from_name") or hostname or "Cabinet"

        # send IP if reminder came directly from outside of server
        client_name = os.getenv('SSH_CONNECTION')

        if client_name:
            client_name = client_name.strip().replace('\n', '').replace('\r', '').split(" ")[0]
        else:
            client_name = ""

        email_from = f'{cab_from_name}<br>{client_name}'

        # Set default `from_name` if unset.
        if from_name is None:
            from_name = f"{cab_from_name} <{self.username}>"

        # Set default `to_addr` if unset.
        if to_addr is None:
            to_addr = self.cab.get("email", "to")

            if to_addr is None:
                self.cab.log("cabinet -> email -> to is unset", level="error")
                return

        # Append `signature` to the `body` of the email.
        signature = signature or f"<br><br>Thanks,<br>{email_from}"
        body += unquote(signature)

        # Create the message object.
        message = MIMEMultipart()
        message["Subject"] = unquote(subject)
        message["From"] = from_name
        message["To"] = (', ').join(to_addr)
        message.attach(MIMEText(unquote(body), "html"))

        if not self.smtp_server:
            self.cab.log("No SMTP Server set", level="error")
            return

        if not self.port:
            self.cab.log("No port set", level="error")
            return

        if not isinstance(self.port, int):
            self.cab.log(f"Port is not an integer (received '{self.port}')", level="error")
            return

        # Send the email.
        server = smtplib.SMTP_SSL(self.smtp_server, self.port)

        try:

            if not self.username or not self.password:
                self.cab.log("Username/password not set", level="error")
                return
            server.login(self.username, self.password)

            server.send_message(message)

            if logging_enabled:
                self.cab.log(
                    f"Sent Email to {message['To']} as {message['From']}: {subject}",
                    is_quiet=is_quiet)

        except smtplib.SMTPAuthenticationError as err:
            self.cab.log(
                f"Could not log into {self.username}; set this with Cabinet.\n\n{err}",
                level="error")

    def send_in_background(self, subject: str, body: str, to_addr: str = '') -> None:
        """
        Sends an email in the background to allow scripts to complete without waiting.

        Args:
            subject (str): The subject line of the email.
            body (str): The main content of the email.
            to_addr (str, optional): The recipient's email address.
                If not provided, defaults to the first address in the cabinet -> email -> to

        Returns:
            None
        """
        # Fetch default to_addr if not provided
        if not to_addr:
            cab_to: List[str] = self.cab.get("email", "to", return_type=list) or []
            to_addr = cab_to[0] if cab_to else ''
            if not to_addr:
                self.cab.log("No email address to send_in_background", level="error")
                return  # Exit if there is no address to send to

        # Sanitize and prepare arguments to avoid shell injection and handle internal single quotes
        subject = shlex.quote(subject)
        body = shlex.quote(body)
        to_addr = shlex.quote(to_addr)

        # Prepare the command string with proper quoting
        command = f"cabinet --mail -s {subject} -b {body}"
        if to_addr:
            command += f" --to {to_addr}"

        # Schedule the command to run immediately using 'at'
        schedule_command = f'echo {command} | at now'
        subprocess.Popen(schedule_command, shell=True,
                        stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)


if __name__ == "__main__":
    if len(sys.argv) == 1:
        print("Usage: cabinet --mail -s <subject> -b <body> --to <to, optional>")
